//CHECKSTYLE:OFF
package tsp.csc4509.tpnote;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static tsp.csc4509.common.Log.TEST;
import static tsp.csc4509.common.Log.GEN;
import static tsp.csc4509.common.Log.COMM;

import java.io.IOException;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import org.apache.logging.log4j.Level;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import tsp.csc4509.blockfile.BlockFile;
import tsp.csc4509.common.Log;
import tsp.csc4509.divers.AppliGenerateur;
import tsp.csc4509.tcp.TcpServerSynchrone;
import tsp.csc4509.tcp.TcpSocketSynchrone;


public class TestClient {
	static final public String TESTFILE = "fichier40k";
	static final public String EMPYFILE="fichierVide";
	static final public int VALUE = 10000;
	static final public int OFFSET = 20000;
	static final public int SIZE = 10000;
	/**
	 * valeur du seed pour que ServerInfo.getRandomServer() retourne le SERVER1
	 */
	static final public long SEEDSERVER1 = 4096;
	/**
	 * valeur du seed pour que ServerInfo.getRandomServer() retourne le SERVER2
	 */
	static final public long SEEDSERVER2 = 0;


	@BeforeClass
	static public void before() throws IOException {
		Log.setLevel(TEST, Level.INFO);
		Log.setLevel(GEN, Level.INFO);
		Log.setLevel(COMM, Level.INFO);

		// mise en place du fichier de test

		String fileName = TESTFILE;
		AppliGenerateur.generateFile(BlockFile.SOURCE + "/" + fileName, VALUE);
		BlockFile.createFile(fileName);

		fileName  = EMPYFILE;
		AppliGenerateur.generateFile(BlockFile.SOURCE + "/" + fileName, 0);
		BlockFile.createFile(fileName);
	}

	/**
	 * Test unitaire de la méthode connectAndSendRequest()
	 *  trois éléments testés:
	 *    1. le canal connecté au serveur est construit;
	 *    2. le canal a été connecté;
	 *    3. la Request a été envoyée reçu à l'identique par le serveur.
	 * @throws IOException
	 * @throws InterruptedException
	 */
	@Ignore
	@Test
	public void testConnectAndSendRequest() throws IOException, InterruptedException {
		TEST.info("[TestClient::testConnectAndSendRequest] début");
		final Request sendRequest = new Request(TESTFILE, OFFSET, SIZE);

		// lancement d'un serveur qui attend une connexion et reçoit la Request
		Thread serveur = new Thread(new Runnable() {
			@Override
			public void run() {
				TEST.info("[TestClient::testConnectAndSendRequest] serveur en route");
				try (
						TcpServerSynchrone serveur = new TcpServerSynchrone(ServerInfo.SERVER2.getPort());
						TcpSocketSynchrone client = serveur.acceptClient();
						) {
					TEST.info("[TestClient::testConnectAndSendRequest] client accepté");
					Request recvRequest = (Request) client.receiveObject();
					assertTrue("La requête reçue est identique à la requête émise", recvRequest.equals(sendRequest));
					Thread.sleep(1000);

				} catch (Exception e) {
					return;
				}
			}

		});
		serveur.start();
		Thread.sleep(1000);

		Client client = new Client(TESTFILE);


		SocketChannel chan = client.connectAndSendRequest(ServerInfo.SERVER2, sendRequest);
		assertTrue("canal non null", chan != null);
		assertTrue("canal connecté", chan.isConnected());
		// le test n'essaie pas d'écrire: le serveur a déjà du recevoir un message
		chan.close();

		// on arrête de serveur si ce n'est pas déjà fait.
		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}

	}

	@Ignore
	@Test
	public void testManageNextBlock() throws IOException, InterruptedException {
		TEST.info("[TestClient::testManageNextBlock] début");
		// lancement d'un serveur qui attend une connexion et reçoit la Request
		Runnable serveurRunnable = new Runnable() {
			@Override
			public void run() {
				TEST.info("[TestClient::testManageNextBlock] serveur en route");
				try (
						TcpServerSynchrone serveur = new TcpServerSynchrone(ServerInfo.SERVER1.getPort());
						TcpSocketSynchrone client = serveur.acceptClient();
						) {
					TEST.info("[TestClient::testManageNextBlock] client accepté");
					Request recvRequest = (Request) client.receiveObject();
					Thread.sleep(1000);

				} catch (Exception e) {
					return;
				}
			}
		};

		Thread serveur = new Thread(serveurRunnable);

		// test de manageNextBlock() quand il ne reste plus de bloc à gérer.
		TEST.info("[TestClient::testManageNextBlock] aucun bloc");
		serveur.start();
		Thread.sleep(1000);
		Client client = new Client(EMPYFILE);

		ServerInfo.chooseRandom(SEEDSERVER1);
		client.manageNextBlock();
		assertTrue("le dictionnaire serverMap est vide", client.getServerMap().size() == 0);

		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}

		// test avec des blocs à gerer
		TEST.info("[TestClient::testManageNextBlock] avec un bloc");
		serveur = new Thread(serveurRunnable);
		serveur.start();
		Thread.sleep(1000);
		client = new Client(TESTFILE);

		ServerInfo.chooseRandom(SEEDSERVER1);
		client.manageNextBlock();
		assertTrue("le dictionnaire serverMap contient 1 élement", client.getServerMap().size() == 1);
		assertTrue("Un canal est enregistré auprès du selector", client.getServerMap().keySet().size() == 1);
		for (SocketChannel chan : client.getServerMap().keySet()) {
			assertTrue("le canal est connecté", chan.isConnected());
			assertFalse("le canal est non bloquant", chan.isBlocking());
			for (SelectionKey key: client.getSelector().keys()) {
				assertTrue("le canal est enregistré auprès du selector", key.channel() == chan);
			}
		}

		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}
	}

	@Ignore
	@Test
	public void testRobustesteManageNextBlock() throws InterruptedException, IOException {
		TEST.info("[TestClient::testManageNextBlock] début");
		// lancement d'un serveur qui attend une connexion et reçoit la Request
		Runnable serveurRunnable = new Runnable() {
			@Override
			public void run() {
				TEST.info("[TestClient::testManageNextBlock] serveur en route");
				try (
						TcpServerSynchrone serveur = new TcpServerSynchrone(ServerInfo.SERVER1.getPort());
						TcpSocketSynchrone client = serveur.acceptClient();
						) {
					TEST.info("[TestClient::testManageNextBlock] client accepté");
					Request recvRequest = (Request) client.receiveObject();
					Thread.sleep(1000);

				} catch (Exception e) {
					return;
				}
			}
		};

		

		// test avec des blocs à gerer mais un serveur absent
		TEST.info("[TestClient::testRobustesteManageNextBlock] Un des serveurs absents");
		Thread serveur = new Thread(serveurRunnable);

		
		serveur = new Thread(serveurRunnable);
		serveur.start();
		Thread.sleep(1000);
		Client client = new Client(TESTFILE);

		ServerInfo.chooseRandom(SEEDSERVER2);
		client.manageNextBlock();
		assertTrue("le dictionnaire serverMap contient 1 élement", client.getServerMap().size() == 1);
		assertTrue("Un canal est enregistré auprès du selector", client.getServerMap().keySet().size() == 1);
		for (SocketChannel chan : client.getServerMap().keySet()) {
			assertTrue("le canal est connecté", chan.isConnected());
			assertFalse("le canal est non bloquant", chan.isBlocking());
			for (SelectionKey key: client.getSelector().keys()) {
				assertTrue("le canal est enregistré auprès du selector", key.channel() == chan);
			}
		}

		if (serveur.isAlive()) {
			serveur.interrupt();
			serveur.join();
		}

	}


}
